﻿/*
 pwrc

	Presentation with realtime comment.

		2010/08 K.Uehara
*/

/*
	定数定義
*/

var PWRC_Conf = {
	TwitSearchURL: "http://search.twitter.com/search.json",

	DelayTick: 64, // システムTick。約4frame.

	DefaultStep: 8,
	FontName: "メイリオ,Osaka",
	FontSize: 40,
	YStep: 48,
	Margin: 48,
	CommentLines: 4,
	CommentNum: 24,

	Twit_UP_Step: 4,
	Twit_BOX_W: 512,
	Twit_BOX_UpShift: 0,
	Twit_Display_Time: 15*1000,
	Twit_Fade_Time: 0.5*1000,
	Twit_Display_Num: 4,
};

PWRC_Conf.SendInterval = Math.round( 5000/ PWRC_Conf.DelayTick ),
PWRC_Conf.Twit_Get_Interval = Math.round( (30*1000) / PWRC_Conf.DelayTick ),
PWRC_Conf.Twit_Disp_Wait = Math.round( (1*1000) / PWRC_Conf.DelayTick ),

/*
	字幕それぞれ
*/

var Caption = function( stage ){
	this.stage = stage;
	this.step = PWRC_Conf.DefaultStep;
	this.is_run = false;

	this.init();
}

Caption.prototype = {
	init : function() {
		// 初期化。表示用spriteを作る。
		var sprite = new window.runtime.flash.display.Sprite();
		sprite.cacheAsBitmap = true;

		var t1;
		t1 = new window.runtime.flash.text.TextFormat();
		t1.font = PWRC_Conf.FontName;
		t1.size = PWRC_Conf.FontSize;
		t1.color = 0xE0E0E0;
		this.textformat = t1;

		var t2;
		t2 = new window.runtime.flash.text.TextFormat();
		t2.font = PWRC_Conf.FontName;
		t2.size = PWRC_Conf.FontSize;
		t2.color = 0x202020;
		this.textformat_o = t2;

		// 袋文字用textfield。0が本体。

		var textfield = new Array();

		for( var i = 4; i >= 0; i-- ){
			var t;

			t = textfield[ i ] = new window.runtime.flash.text.TextField();
			t.text = "";
			sprite.addChild( t );
		}

		textfield[ 1 ].x = -1;
		textfield[ 2 ].x = +1;
		textfield[ 3 ].y = -1;
		textfield[ 4 ].y = +1;

		this.textfield = textfield;

		this.sprite = sprite;

		this.stage.addChild( sprite );
	},
	
    set : function( str, y ){
		// 袋文字処理
		for( var i = 0; i < 5; i++ ){
			var t = this.textfield[ i ];

			t.text = str;

			if( i == 0 ){
				t.setTextFormat( this.textformat );
			} else {
				t.setTextFormat( this.textformat_o );
			}

			t.width = t.textWidth+16;
			t.height = t.textHeight+16;

			this.width = t.width;
		}

		this.sprite.x = this.stage.stageWidth;
		this.sprite.y = y;

		this.is_run = true;
	},
	
	is_next_ok : function() {
		var margin = PWRC_Conf.Margin;
		if( this.is_run && this.sprite.x + this.width + margin < this.stage.stageWidth ){
			return true;
		} else {
			return false;
		}
	},
	  
	next : function() {
		if( this.is_run == false ){
			return false;
		}

		this.sprite.x -= this.step;

		if( this.sprite.x < -this.width ){
			this.is_run = false;
			for( var i = 0; i < 5; i++ ){
				this.textfield[ i ].text = "";
			}
			this.sprite.x = this.stage.stageWidth;
			return false;
		}
		return true;
	},
}

/*
	マネージャー
*/

var Manager = function( stage, msg_id ) {
	this.delay = PWRC_Conf.DelayTick;
	this.mesg = new Array();

	var line_num = PWRC_Conf.CommentLines;
	var cap_num = PWRC_Conf.CommentNum;

	this.lines = new Array();
	for( var i = 0; i < line_num; i++ ){
		this.lines[ i ] = new Array();
		this.lines[ i ].pos = i * PWRC_Conf.YStep;
		this.lines[ i ].caps = new Array();
	}

	this.caps = new Array();
	for( var i = 0; i < cap_num; i++ ){
		var c = new Caption( stage );
		this.caps.push( c );
	}

	this.sending = 1;
	this.msg_id = msg_id;

	this.is_pause = false;

	this.init();
}

Manager.prototype = {
	init : function() {
		this.run();
	},
	next : function() {

		if( this.is_pause ){
			return;
		}
		for( var i in this.lines ){
			var next_ok = true;
			if( this.lines[ i ].caps.length > 0 ){
				var n = new Array();
				for( var j in this.lines[ i ].caps ){
					var c = this.lines[ i ].caps[ j ];
					var r = c.next();
					if( r ){
						n.push( c );
						next_ok = c.is_next_ok();
					} else {
						this.caps.push( c );
					}
				}
				this.lines[ i ].caps = n;
			}
			if( next_ok ){
				if( this.mesg.length > 0 && this.caps.length > 0 ){
					var str = this.mesg.shift();
					var c = this.caps.pop();
					c.set( str, this.lines[ i ].pos );
					this.lines[ i ].caps.push( c );

					if( this.callback != undefined ){
						this.callback( str );
					}
					
					break;
				}
			}
		}

		if( this.sending > 0 ){
			this.sending --;
		}
		if( this.sending == 0 && this.url != "" ){
			var req = new XMLHttpRequest();
			var self = this;

			req.open( 'GET', this.url + "?id="+this.msg_id, true );
			req.onreadystatechange = function( aEvt ){
				if( req.readyState == 4 ){
					if( req.status == 200 ){
						/* 取得できた */
						self.parse( req.responseText );
					}
					self.sending = PWRC_Conf.SendInterval;
				}
			}
			req.send( null );
			this.sending = -1;
		}
	},
	run : function() {
		this.timer_id = window.setTimeout( function( self ){ self.next(); self.run(); }
											, this.delay, this );
	},
	set: function( str ){
		this.mesg.push( str );
	},

	parse : function( str ){
		var res = eval( '('+ str + ')' );
		for( var i = 0, len = res.data.length; i < len; i++ ){
			if( this.msg_id < res.data[ i ].id ){
				this.msg_id = res.data[ i ].id;
			}
			var msg = res.data[ i ].message;

			this.set( msg );
		}
	},

	pause : function( flag ){
		this.is_pause = flag;
	},

	y_shift : function( ytop ){
		for( var i = 0, len = this.lines.length; i < len; i++ ){
			this.lines[ i ].pos = (i * PWRC_Conf.YStep) + ytop;
		}
	},

	set_callback : function( func ) {
		this.callback = func;
	}
}


/*
	twitter 表示
*/

var TwitBox = function( stage ){
	this.stage = stage;
	this.step = PWRC_Conf.Twit_UP_Step;

	this.init();
}

TwitBox.prototype = {
	init: function() {
		this.rect = new window.runtime.flash.display.Sprite();
		this.textfield = new window.runtime.flash.text.TextField();
		this.textformat = new window.runtime.flash.text.TextFormat();
		this.textformat.font = PWRC_Conf.FontName;

		this.rect.addChild( this.textfield );
		this.stage.addChild( this.rect );

		this.rect.x = 0;
		this.rect.y = this.stage.stageHeight;
		this.rect.cacheAsBitmap = true;

		var t = this.textfield;
		t.x = 4;
		t.y = 4;
		t.width = PWRC_Conf.Twit_BOX_W - 8;
		t.height = 512;

		this.is_run = false;
	},
	set: function( date, from, mesg ) {
		var t = this.textfield;
		t.htmlText =
		    '<font size="16" color="#87CEFA">'+date
			+"   "+from+"\n</font>"
			+'<font size="20" color="#FFFFFF">'+mesg+"</font>";
		t.wordWrap = true;
		t.setTextFormat( this.textformat );

		t.height = t.textHeight + 8;

		var child = this.rect;

		child.graphics.beginFill( 0x000000, 0.6 );
		child.graphics.lineStyle( 0, 0x666666 );
		child.graphics.drawRoundRect( 0, 0, t.width + 4, t.height, 16 );
		child.graphics.endFill();

		this.rect.x = this.stage.stageWidth - t.width - 4;
		this.rect.y = this.stage.stageHeight - PWRC_Conf.Twit_BOX_UpShift;

		this.duration = PWRC_Conf.Twit_Display_Time;

		this.is_run = true;
		this.rect.alpha = 1.0;
	},
	move: function( step ) {
		this.rect.y -= step;
	},
	run: function( tick ) {
		if( ! this.is_run ){
			return;
		}
		this.duration -= tick;

		if( this.duration <= 0 ){
			this.clear();
			this.is_run = false;
		} else if( this.duration < PWRC_Conf.Twit_Fade_Time ){
			this.rect.alpha = ( this.duration ) / ( PWRC_Conf.Twit_Fade_Time );
		}
	},
	clear: function() {
		this.textfield.htmlText = "";
		this.rect.graphics.clear();
		this.rect.alpha = 1.0;
		this.is_run = false;
	},
	is_need_up: function() {
		if( this.rect.y + this.rect.height < this.stage.stageHeight - PWRC_Conf.Twit_BOX_UpShift ){
			return false;
		}
		return true;
	},
}

var TwitMan = function( stage ) {
	this.stage = stage;
	this.init();
	this.run();
}

TwitMan.prototype = {
	reset_query: function( query ) {
		if( this.query != query ){
			this.query = query;
			this.getparam = "?q=" + query + "&rpp=1";
		}
		if( this.sending > 1 ){
			this.sending = 1;
		}
	},
	init: function( stage ) {
		this.delay = PWRC_Conf.DelayTick;
		this.disp_wait = 0;

		this.twitboxes = new Array();
		for( var i = 0; i < PWRC_Conf.Twit_Display_Num; i++ ){
			var t = new TwitBox( this.stage );
			this.twitboxes.push( t );
		}
		this.sending = 100;

		this.twits = new Array();
		this.moving = null;

		this.lasttime = window.runtime.flash.utils.getTimer();

		this.is_pause = false;

		this.url = "";
	},

	next: function() {
		// 画面更新処理

		var tick;

		var now = window.runtime.flash.utils.getTimer();
		tick = ( now - this.lasttime );
		this.lasttime = now;

		if( this.is_pause ){
			return;
		}

		if( this.moving == null ){
			if( this.disp_wait > 0 ){
				this.disp_wait --;
			} else if( this.twits.length > 0 ){
				var data = this.twits.pop();
				var box = this.twitboxes.shift();

				box.clear();
				box.set( data.date, data.from, data.msg );

				if( this.callback != undefined ){
					this.callback( data.date+" "+data.from+": "+data.msg );
				}
				
				this.twitboxes.push( box );
				this.moving = box;
			}
		} else {
			for( var i in this.twitboxes ){
				this.twitboxes[ i ].move( PWRC_Conf.Twit_UP_Step );
			}
			if( ! this.moving.is_need_up() ){
				this.moving = null;
				this.disp_wait = PWRC_Conf.Twit_Disp_Wait;
			}
		}

		for( var i in this.twitboxes ){
			this.twitboxes[ i ].run( tick );
		}

		// 取得処理

		if( this.sending > 0 ){
			this.sending --;
		}
		if( this.sending == 0 && this.getparam != "" ){
			var req = new XMLHttpRequest();
			var self = this;

			req.open( 'GET', PWRC_Conf.TwitSearchURL + this.getparam, true );
			req.onreadystatechange = function( aEvt ){
				if( req.readyState == 4 ){
					/* 読み込み完了時 */
					if( req.status == 200 ){
						/* 取得できた */
						self.parse( req.responseText );
					}
					self.sending = PWRC_Conf.Twit_Get_Interval;
				}
			}
			req.send( null );
			this.sending = -1;
		}
	},
	run : function() {
		this.timer_id = window.setTimeout( function( self ){ self.next(); self.run(); }
 											, this.delay, this );
	},

	parse : function( str ){
		var res = eval( '('+ str + ')' );
		while( res.results.length > 0 ){
			var d;
			d = res.results.pop();
			var date = new Date( d.created_at );
			var datestring = date.toLocaleTimeString();
			var data = {
				msg: d.text,
				from: d.from_user,
				date: datestring,
			};
			this.twits.unshift( data );
		}
		this.getparam = res.refresh_url;
	},

	pause : function( flag ) {
		this.is_pause = flag;
	},

	set_callback : function( func ) {
		this.callback = func;
	}

}

/* 本体 */

var PWRC = function() {
	this.newWindow = null;
 	this.manager = null;
	this.twitter = null;

	this.is_pause = false;
}

PWRC.prototype = {
	window_init : function( disp_id ){
		var newWindow;
		var options = new air.NativeWindowInitOptions();
		options.transparent = true;
		options.systemChrome = air.NativeWindowSystemChrome.NONE;

		options.type = air.NativeWindowType.NORMAL;

		newWindow = new air.NativeWindow( options );

		newWindow.title = "PWRC overlay";

		var screens = air.Screen.screens;
		if( disp_id >= screens.length ){
			disp_id = screens.length - 1;
		}

		newWindow.x = screens[ disp_id ].bounds.x;
		newWindow.y = screens[ disp_id ].bounds.y;
		newWindow.width = screens[ disp_id ].bounds.width;
		newWindow.height = screens[ disp_id ].bounds.height;

		newWindow.stage.scaleMode = window.runtime.flash.display.StageScaleMode.NO_SCALE;
		newWindow.stage.align = window.runtime.flash.display.StageAlign.TOP_LEFT;

		newWindow.alwaysInFront = true;

		newWindow.activate();

		return newWindow;
	},

	start : function( disp_id, msg_id, comment_flag, twitter_flag ) {
		this.newWindow = this.window_init( disp_id );

		if( comment_flag ){
			this.manager = new Manager( this.newWindow.stage, msg_id );
		}
		if( twitter_flag ){
			this.twitter = new TwitMan( this.newWindow.stage );
		}
		return 0;
	},

	end : function() {
		air.NativeApplication.nativeApplication.exit(0);
	},

	pause_toggle : function() {
		this.is_pause = !this.is_pause;
		if( this.manager != null ){
			this.manager.pause( this.is_pause );
		}
		if( this.twitter != null ){
			this.twitter.pause( this.is_pause );
		}
		return this.is_pause;
	},

	y_shift_comment : function( ytop ){
		if( this.manager != null ){
			this.manager.y_shift( parseInt(ytop) );
		}
	},

	set_comment_url : function( url ){
		if( this.manager != null ){
			this.manager.url = url;
		}
	},

	set_twit_search_string : function( str ){
		if( this.twitter != null ){
			this.twitter.reset_query( str );
		}
	},

	set_twit_box_y_shift : function( y ) {
		PWRC_Conf.Twit_BOX_UpShift = y;	
	},

	set_comment_callback : function( func ) {
		if( this.manager != null ){
			this.manager.set_callback( func );
		}
	},
	set_twitter_callback : function( func ) {
		if( this.twitter != null ){
			this.twitter.set_callback( func );
		}
	},
}

